import fs from 'fs'
import path from 'path'
import crypto from 'crypto'

interface IFileConstructorOption {
	name: string
	type: string
	saveDir: string
}

/**
 * 表单提交的文件
 */
export class File {
	/** 文件名 */
	public readonly name: string
	/** mime类型 */
	public readonly type: string
	/** 文件保存路径 */
	public path: string

	#ws: fs.WriteStream
	#hash: crypto.Hash
	#hashstr!: string
	#finished = false
	#size = 0
	#top64 = Buffer.from('')

	constructor(option: IFileConstructorOption) {
		this.name = option.name
		this.type = option.type
		const randomText = parseInt(Math.random() * 10000 as any).toString().padStart(4, '0')
		this.path = path.join(option.saveDir, `${Date.now()}${randomText}`)
		this.#ws = fs.createWriteStream(this.path)
		this.#hash = crypto.createHash('md5')
	}

	/**
	 * 写入数据到文件
	 * @param data 写入的数据
	 */
	public async write(data: Buffer) {
		if (this.#finished || !this.#ws.writable) return
		//开始64字节
		if (this.#top64.length < 64) {
			this.#top64 = Buffer.concat([
				this.#top64,
				data.slice(0, 64 - this.#top64.length)
			])
		}

		this.#size += data.length

		//写入
		await Promise.all([
			new Promise<void>((resolve, reject) => this.#ws.write(data, err => err ? reject(err) : resolve())),
			new Promise<void>((resolve, reject) => this.#hash.write(data, err => err ? reject(err) : resolve())),
		])
	}

	/**
	 * 结束
	 */
	public finish() {
		this.#ws.destroy()
		this.#hashstr = this.#hash.digest('hex')
		this.#hash.destroy()
		this.#finished = true
	}

	/** 文件hash */
	public get hash() {
		return this.#hashstr
	}

	/** 文件大小 */
	public get size() {
		return this.#size
	}

	/** 文件开始64字节 */
	public get top64() {
		return this.#top64
	}
}